/**************************************************
 * The source code of generating profile-based proteins. 
 This script is dependent on PSI-BLAST package. Make sure
 the PSI-BLAST package and the script were in the same folder . 
 This script was written by Bin Liu <bliu@insun.hit.edu.cn> and 
 Qiwen Dong <qwdong@insun.hit.edu.cn>
 **************************************************/
#include "psi_freq.h"
#include "rdfasta.h"
#include "rd_pdb.h"
#include "direct.h"
#include "copyfile.h"
#include "amino_acid.h"
#include <io.h>
#pragma warning(disable : 4996)
typedef struct
{
	double value;
	int label;
} sample;


bool Psi_Freq_hom(string homdir,string profile_dir,string& errstr);
string bryant2astral(string br);
void ToLower(string& str);
bool Profile_Exist(string sprofile,string pdbchain);
void Convertu2x(string& tempseq);
void Convertx2u(string& tempseq);
bool Profile2Seq(string id,string profile_dir,int N,string& profile_seq);
void sortprofile(vector<sample>& all_sample);
string pos2aa( int pos);



void main(int argv,char * argc[])
{
	string inputfilename;
	
	string blastseqfile,sprofile,sname,dname;
	string psi_outname;
	string psi_pssmname;
	string profile_dir;	
	string line,sfname,dfname;
	string temfilenameblaststring,seq,seq1,sec,miaoshu,tempseq;
	string tempdbchain,pdbchain,temfilename,blaststring;	
	vector<string> all_pdb;
	vector<string> miss_pdb;
	vector<string> err;
	vector<vector<double> > profile;
	string profile_seq;
	string profile_seq_file_name;
	int i;	
	int top;

	if(argv<3)
	{
		cout<<"usage: [program name] [input protein sequence file in FASTA format] [frequency profile output folder] [type of profile-based proteins (1=p1, 2=p2, ..., 20=p20) ][profile-based protein output file]"<<endl;
		return;
	}
	inputfilename=argc[1];
	profile_dir=argc[2];	
	top=atoi(argc[3]);
	blastseqfile=inputfilename+".seq";
	psi_outname=inputfilename+".resu";
	psi_pssmname=inputfilename+".pssm";

	ofstream fileout_profile(argc[4]);
	if(!fileout_profile){
		cout<<"cannot open output file "<<argc[4]<<endl;
		return ;
	}
	_mkdir(profile_dir.c_str());
	//reading input file
	ifstream filein(inputfilename.c_str());
	if(!filein.is_open())
	{
		cout<<"cannot open file "<<inputfilename.c_str()<<endl;
		return ;
	}
	int count=0;
	int start,end;
	while(ReadFastaSeq(filein,seq,miaoshu))
	{	
		Upper(seq);
		
		pdbchain.clear();
		count++;
		
		start=0;
		end=start;
		while((end<(int)miaoshu.length())&&(miaoshu[end]!=' ')&&(miaoshu[end]!='\t')&&(miaoshu[end]!='|'))
			end++;
		pdbchain=miaoshu.substr(start,end-start);
		cout<<"Processing sequence"<<count<<":"<<pdbchain.c_str()<<"...";
		{				
			if(seq.length()<=30)
			{
				miss_pdb.push_back(pdbchain);
				err.push_back("protein length < 30 AA");
				cout<<"protein length < 30 AA"<<endl;
				WriteFastaSeq(fileout_profile,seq,pdbchain);
				continue;
			}			

			
			if(!WriteFastaSeq(blastseqfile,seq))
			{
				miss_pdb.push_back(pdbchain);
				err.push_back("cannot write sequence into file");
				cout<<"cannot write sequence into file"<<endl;
				continue;
			}
			//You can change the parameters of PSI-BLAST by modifying the following code
			blaststring="blastpgp -d nr -m 1 -j 10 -e 0.001 -i "+blastseqfile+" -o "+psi_outname+" -Q "+psi_pssmname; 
			
			
			system(blaststring.c_str());
			if(!psi_freq(psi_outname,profile,tempseq,seq))
			{
				miss_pdb.push_back(pdbchain);
				err.push_back("error when calculating the frequency profile");	
				cout<<"error when calculating the frequency profile"<<endl;
				WriteFastaSeq(fileout_profile,seq,pdbchain);
				continue;
			}
			if(seq!=tempseq)
			{	
				miss_pdb.push_back(pdbchain);
				err.push_back("template sequence and target sequence are different");
				cout<<"template sequence and target sequence are different"<<endl;
				cout<<seq.c_str()<<endl;
				cout<<tempseq.c_str()<<endl;
				WriteFastaSeq(fileout_profile,seq,pdbchain);
				continue;			
			}
			if(!write_profile(pdbchain,profile,seq,1,profile_dir))
			{
				miss_pdb.push_back(pdbchain);
				err.push_back("error when writing frequency profile");
				cout<<"error when writing frequency profile"<<endl;
				continue;
			}
			profile_seq.clear();
			if(!Profile2Seq(pdbchain, profile_dir, top ,profile_seq)){
				miss_pdb.push_back(pdbchain);
				err.push_back("error when converting frequency profile into profile-based protein");
				cout<<"error when converting frequency profile into profile-based protein: "<<pdbchain<<endl;
				WriteFastaSeq(fileout_profile,seq,pdbchain);
				continue;
			}
			WriteFastaSeq(fileout_profile,profile_seq,pdbchain);
				
			if(!psi_log(psi_pssmname,profile,tempseq,seq))
			{
				miss_pdb.push_back(pdbchain);
				err.push_back("error when reading PSSM");
				cout<<"error when reading PSSM"<<endl;
				continue;
			}
			seq1=seq;
			Convertx2u(seq1);
			if(seq1!=tempseq)
			{
				miss_pdb.push_back(pdbchain);
				err.push_back("template sequence and the target sequence are different");
				cout<<"template sequence and the target sequence are different"<<endl;
				cout<<seq1.c_str()<<endl;
				cout<<tempseq.c_str()<<endl;				
				continue;			
			}
			if(!write_profile(pdbchain,profile,seq,2,profile_dir))
			{
				miss_pdb.push_back(pdbchain);
				err.push_back("error when writing PSSM");
				cout<<"error when writing PSSM"<<endl;
				continue;
			}
			cout<<"successful"<<endl;
		}
		
	}
	if(miss_pdb.size()<=0)
	{
		cout<<"All successful"<<endl;
	}
	else
	{
		cout<<"some sequence cannot generate frequency profiles, read log.txt file for more information"<<endl;
        ofstream fileout("log.txt");
		if(!fileout.is_open())
			cout<<"cannot open log.txt"<<endl;
		else
		{
			for(i=0;i<(int)miss_pdb.size();i++)
				fileout<<miss_pdb[i]<<'\t'<<err[i]<<endl;
		}
	}	
}

void ToLower(string& str)
{
	for(int i=0;i<(int)str.length();i++)
	{
		if(isalpha(str[i]))
			str[i]=tolower(str[i]);
	}
}
bool Profile_Exist(string sprofile,string pdbchain)
{
	string sfname,dfname;
	intptr_t ptr;
	_finddata_t data;

	sfname=sprofile+"\\"+pdbchain+".frp";
	dfname=sprofile+"\\"+pdbchain+".lop";
	ptr=_findfirst(sfname.c_str(),&data);
	if(ptr==-1)
	{		
		return false;
	}
	_findclose(ptr);
	ptr=_findfirst(dfname.c_str(),&data);
	if(ptr==-1)
		return false;
	_findclose(ptr);
	return true;
}
void Convertu2x(string& tempseq)
{
	int i,len;
	len=(int)tempseq.length();
	for(i=0;i<len;i++)
	{
		if(tempseq[i]=='u'||tempseq[i]=='U')
			tempseq[i]='X';
	}
}
void Convertx2u(string& tempseq)
{
	int i,len;
	len=(int)tempseq.length();
	for(i=0;i<len;i++)
	{
		if(tempseq[i]=='x'||tempseq[i]=='X')
			tempseq[i]='U';
	}
}



bool Profile2Seq(string id,string profile_dir,int N,string& profile_seq)
{
	vector<vector<double> > profile;
	string seq;
	//vector<int> one_profile;
	int i,len;
	//string index;
	if(!read_profile(id,profile,seq,1,profile_dir))
		return false;
	len=(int)profile.size();

	for(i=0;i<len;i++)
	{


		vector<sample> all_sample;
		for(int k=0;k<20;k++){
			sample tempsample;
			tempsample.label=k;
			tempsample.value=profile[i][k];
			all_sample.push_back(tempsample);
		}
		sortprofile(all_sample);
		//int position=all_sample[N-1].label;
		profile_seq=profile_seq+pos2aa( all_sample[N-1].label);
	}
	return true;
}
void sortprofile(vector<sample>& all_sample)
{
	int i,j,len;
	sample temp;
	len=(int)all_sample.size();
	for(i=len-1;i>0;i--)
	{
		for(j=0;j<i;j++)
		{
			if(all_sample[j].value<all_sample[j+1].value)
			{
				temp=all_sample[j];
				all_sample[j]=all_sample[j+1];
				all_sample[j+1]=temp;
			}
		}
	}
}

string pos2aa( int pos)
{
	string rno;
	//cno  = tolower(c);
	
	switch (pos)
	{
	case 0 : rno = "a"; break;
    case 1: rno = "c"; break;
    case 2: rno = "d"; break;
    case 3: rno = "e"; break;
    case 4: rno = "f"; break;
    case 5: rno = "g"; break;
    case 6: rno = "h"; break;
    case 7: rno = "i"; break;
    case 8: rno = "k"; break;
    case 9: rno = "l"; break;
    case 10: rno = "m"; break;
    case 11: rno = "n"; break;
    case 12: rno = "p"; break;
    case 13: rno = "q"; break;
    case 14: rno = "r"; break;
    case 15: rno = 's'; break;
    case 16: rno = 't'; break;
    case 17: rno = 'v'; break;
    case 18: rno = 'w'; break;
    case 19: rno = 'y'; break;
    default: rno = "x";
	}
	return rno;
}