#ifndef _RD_PDB_H
#define _RD_PDB_H

#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <sstream>
#include <conio.h>
#include <math.h>
#include "amino_acid.h"
#include "direct.h"
#include "io.h"

using namespace std;
#define NOSEQNUM -9999	
#define NOCHAIN -1		

typedef struct S_Contact
{
	int i1;
	int i2;
} Contact;

class Point
{
public:
	Point();
public:
	double x,y,z;
};

class Atom
{
public:
	Atom();

	Point pt;					
	string name;			
	int chain_seqm;				
	string residule_name;		
	char c_resname;				
	int residule_chain_seqm;	
	char chain_identifier;		
	double occupancy;			
	double tempFactor;			
	string segID;				
	string element;				
	string charge;				

	int residule_seqm;			

	
	double radium;				
};

class Residue
{
public:
	Residue();
	void Clear();
public:
	
	string name;				
	char c_name;				
	int chain_seqm;				
	int num_atom;				
	vector<int> m_atoms;		
	int atom_index[NUM_ATOM_TYPE];	

	
	double alpha,phi,psi,omega,chi_1,chi_2,chi_3,chi_4,chi_5;
	
	int num_contact;

	
	double main_radium;			
	Point main_center;			
	double side_radium;			
	Point side_center;			
	

};
class Chain
{
public:
	Chain();
	void Clear();
	int AtomIndexInAA(int aa_num,string atom_name);//

	
	int ObtainContact(vector<Contact>& all_contact);	
	int SaveContact(const string& filename, const vector<Contact>& all_contact);	

	
	bool CalculateDihedral();
	bool WriteDihedral(const string filename);
	bool Pre_Dihedral(vector<vector<double> >& preference);

	
	bool Cal_Num_Contact();
	bool Pre_Num_Contact(vector<vector<double> >& preference);
	
	bool Rebuildxyz_Dihedral();
public:
	
	char chain_identifier;		
	vector<Residue> m_residues;	
	vector<Atom> m_atoms;		
	int num_atom;				
	int num_residue;			
	string seq;					

	

};
class Protein
{
public:
	Protein();
	void Clear();					
	bool LoadFromPDB(string filename);
	bool SaveToFile(string filename);
	bool SaveChains(string dirname="AUTO",string filename="AUTO");
	bool WriteSeqFasta(string filename);

	int Get_Num_chain();
	void Get_PDBID(string& id);
	bool GetSeq(int chain,string& seq);

	
	int ObtainContact(int chainnumber,vector<Contact>& all_contact);	
	int SaveContact(int chainnumber,const string& filename, const vector<Contact >& all_contact);	

private:
	
	void process_header(const string& line);
	void process_source(const string& line);
	void process_atom(const string& line,vector<Atom>& all_atom);
	void process_field(const string& line);
	bool AddAtom(const vector<Atom>& all_atom);
	void Atom2Line(const Atom& atom,string& line);
	void Atom2Ter(const Atom& atom,string& line);

	void OutputPreField(ofstream& file);
	void OutputSucField(ofstream& file);
	


public:
	
	string PDB_id;
	vector<Chain> m_chains;		
	int num_chain;				

	vector<string> header;		
	vector<string> title;		
	vector<string> compnd;
	vector<string> source;
	vector<string> author;
	vector<string> revdat;
	vector<string> jrnl;
	vector<string> remark;
	vector<string> dbref;
	vector<string> seqadv;
	vector<string> seqres;
	vector<string> het;
	vector<string> hetnam;
	vector<string> formul;
	vector<string> helix;
	vector<string> sheet;
	vector<string> link;
	vector<string> cispep;
	vector<string> origx1;
	vector<string> origx2;
	vector<string> origx3;
	vector<string> scale1;
	vector<string> scale2;
	vector<string> scale3;
	vector<string> hetatm;
	vector<string> concet;
	vector<string> master;
	vector<string> anisou;
	vector<string> cryst1;
	vector<string> expdta;
	vector<string> ftnote;
	vector<string> hetsyn;
	vector<string> hydbnd;
	vector<string> jnrl;
	vector<string> keywds;
	vector<string> modres;
	vector<string> mtrix1;
	vector<string> mtrix2;
	vector<string> mtrix3;	
	vector<string> site;
	vector<string> sltbrg;
	vector<string> sprsde;
	vector<string> ssbond;
	vector<string> turn;
	vector<string> tvect;
	vector<string> conect;
	vector<string> end;


	
};
bool IsDigit(string str);
#endif